#!/data/oir/bin/ksh
#
# SHIFTAP
#
# using input reference aperture file, add the offset to the center 
# (and high, low) parameters, write out modified aperture file
#
# Usage
#  
#   shiftap [options]
#     -i <input file>   default standard in
#     -o <output path>  default ./newaperture - can be full directory path
#                           (intervening dirs are made on the fly), or 
#                           "-" for standard out
#     -s <filename>     file of aperatures and shifts, one set per line
#     -x <x-shift>      shift of aperture in x direction
#     -y <y-shift>      shift of aperture in y direction
#     -v                verbose mode - summary to stderr
#     -vv               very verbose - each ap modification as well to dstderr
#   -x or -u            print the usage info and arg list with current values
#

tmp=/tmp/$$.sap

trap "/bin/rm -rf $tmp" EXIT

mkdir $tmp 2>/dev/null

input=-
output=./newaperture
xshift=0.0
yshift=0.0
fshift=""
verbose=0
isfile=0

while getopts ":i:o:s:x:y:vhu" fval ; do
   case $fval in
     i) input=$OPTARG;;
     o) output=$OPTARG;;
     s) fshift=$OPTARG
        if [[ ! -s $fshift || ! -r $fshift ]]; then
          print -u2 "File \"${fshift}\" does not exist or is not readable";
          exit 1
        fi;;
     x) xshift=$OPTARG;;
     y) yshift=$OPTARG;;
     v) verbose=$((verbose + 1));;
   h|u) print  "Usage: $(basename $0) [options]"
        printf  "          -i %s\tinput file (\"-\" for stdout)\n" $input
        printf  "          -o %s\toutput file (\"-\" for stdout)\n" $output
        if [[ -s $fshift ]]; then
          printf  "          -s %s\tfile of (aps,shifts) to add\n" $fshift
        else
          printf  "          -s %s\tfile of (aps,shifts) to add\n" "<file>"
        fi
        printf  "          -x %.3f\tx-shift to add to all apps\n" $xshift
        printf  "          -y %.3f\ty-shift to add to all apps\n" $yshift
        printf  "          -v (%s)\tverbose - output summary on stderr\n" \
                   $(if [[ $verbose -gt 0 ]]; then echo yes; else echo no;fi)
        printf  "         -vv (%s)\tvery verbose: show each ap line change\n"\
                   $(if [[ $verbose -gt 1 ]]; then echo yes; else echo no;fi)
        printf  "    -h or -u \t\tprint this usage and exit\n"
        printf  "\n"
        exit 1;;
    \?) print -u2 "Unknown option $OPTARG"; exit 1;;
     :) print -u2 "-$OPTARG requires an argument"; exit 1;;
   esac
done

shift $((OPTIND - 1))

# if file of apertures and shift values
if [[ -f $fshift ]]; then  
  tmpfile=$tmp/list
  isfile=1  
  sort -b -k 1n,2 <$fshift >$tmpfile
fi    

dir=$(dirname $output)
mkdir -p $dir 2>/dev/null

[[ $input != "-" ]]  && exec 0<$input
[[ $output != "-" ]] && exec 1>$output 

nawk '
  BEGIN {vb = '$verbose'
         if('$isfile' > 0) {
             xshiftarr[0] = 0.0
             yshiftarr[0] = 0.0
             while (getline < "'$tmpfile'" > 0) {
               if( $0 ~ /^[ \t]*\#/ || $0 ~ /^[ \t]*$/ ) { continue }
               xshiftarr[$1] = $2
               yshiftarr[$1] = $3
               if(vb >= 2) {
                   printf "%3d: %9.3f %9.3f\n", \
                    $1, xshiftarr[$1],yshiftarr[$1] >"/dev/stderr"
               }
             }
             close("'$tmpfile'")
         } else {
             xshiftarr[0] = '$xshift'
             yshiftarr[0] = '$yshift'
         }     
        }
  /^begin/ { ap = $4     
             xcen = $5
             ycen = $6
             if('$isfile' > 0 && ap in xshiftarr) {
                    xshift = xshiftarr[ap]; yshift = yshiftarr[ap] 
               } else {
                    xshift = xshiftarr[0];  yshift = yshiftarr[0]
               }
             $5 = xcen + xshift
             $6 = ycen + yshift
             if(vb) {          # track some numbers for verbose output
               totap += 1   
               if(minap == 0 || ap < minap) minap = ap
               if(maxap == 0 || maxap < ap) maxap = ap
               if(vb >= 2) 
                 { printf "  Ap %3d: old %.3f %.3f  new %.3f %.3f\n", \
                       ap, xcen, ycen, $5, $6  >"/dev/stderr"
                 }
             }
             print; next
           }
  /(center|low|high)[^_]/ { # "low", "high", not "low_reject", "high_reject"
           $2 += xshift
           $3 += yshift
           printf "\t%s\t%s\n", $1, sprintf("%s %s", $2,$3) 
           next 
         }
  {print}
  END { if(vb && 'isfile' > 0) {
           printf " Applied (%.3f,%.3f) pixel offset to %d apertures (%d - %d)\n", \
                xshift, yshift, totap, minap, maxap >"/dev/stderr"
        }
        if(vb && 'isfile' == 0) {
           printf " Applied file %s of offsets to %d apertures (%d - %d)\n", \
              "'$fshift'", totap, minap, maxap >"/dev/stderr"
        }
      }
'
